#! /bin/bash
#/*-----------------------------------------------------------------*/
#/*! 
#  \file c_sonarqube.sh
#  \brief sonqarqube analysis. 
#  \author  M. Gastineau 
#           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 
#
#   Copyright, 2024, CNRS
#   email of the author : Mickael.Gastineau@obspm.fr
#  
#*/
#/*-----------------------------------------------------------------*/
#
#/*-----------------------------------------------------------------*/
#/* License  of this file :
#  This file is "triple-licensed", you have to choose one  of the three licenses 
#  below to apply on this file.
#  
#     CeCILL-C
#     	The CeCILL-C license is close to the GNU LGPL.
#     	( http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html )
#   
#  or CeCILL-B
#        The CeCILL-B license is close to the BSD.
#        (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
#  
#  or CeCILL v2.1
#       The CeCILL license is compatible with the GNU GPL.
#       ( http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.html )
#  
# 
# This library is governed by the CeCILL-C, CeCILL-B or the CeCILL license under 
# French law and abiding by the rules of distribution of free software.  
# You can  use, modify and/ or redistribute the software under the terms 
# of the CeCILL-C,CeCILL-B or CeCILL license as circulated by CEA, CNRS and INRIA  
# at the following URL "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-C,CeCILL-B or CeCILL license and that you accept its terms.
# */
# /*-----------------------------------------------------------------*/
# requires the shell variable : SONARSCANNERVERSION

apt -y update && apt -y upgrade
apt -y install gcovr unzip wget
wget -q https://binaries.sonarsource.com/Distribution/sonar-scanner-cli/sonar-scanner-cli-$SONARSCANNERVERSION-linux.zip
unzip sonar-scanner-cli-$SONARSCANNERVERSION-linux.zip

export WCPPFLAGS="-O0 -g -Wall -Wextra --coverage"

sed -i "s/CALCEPH_SONARSERVER/${CALCEPH_SONARSERVER}/g" sonar-project.properties
sed -i "s/CALCEPH_PROJECTKEY/${CALCEPH_PROJECTKEY}/g" sonar-project.properties
sed -i "s/CALCEPH_LOGIN/${CALCEPH_LOGIN}/g" sonar-project.properties

mkdir build
cd build
ln -s ../sonar-project.properties 
cmake -DCMAKE_C_FLAGS="$WCPPFLAGS" .. && make && make test
../jenkins/extra_check.sh
cd ..
gcovr --gcov-ignore-parse-errors --cobertura-pretty --exclude-unreachable-branches --print-summary  -r src --exclude-directories tests --exclude-directories examples --object-directory=build -k --cobertura -o report.xml
sonar-scanner-$SONARSCANNERVERSION-linux/bin/sonar-scanner -Dsonar.verbose=true
