/*-----------------------------------------------------------------*/
/*! 
  \file calcephthread.h 
  \brief portable thread functions.

  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2006, CNRS
   email of the author : Mickael.Gastineau@obspm.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This file is "triple-licensed", you have to choose one  of the three licenses 
 below to apply on this file.
 
    CeCILL-C
    	The CeCILL-C license is close to the GNU LGPL.
    	( http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html )
   
 or CeCILL-B
        The CeCILL-B license is close to the BSD.
        (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
  
 or CeCILL v2.1
      The CeCILL license is compatible with the GNU GPL.
      ( http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.html )
 

This library is governed by the CeCILL-C, CeCILL-B or the CeCILL license under 
French law and abiding by the rules of distribution of free software.  
You can  use, modify and/ or redistribute the software under the terms 
of the CeCILL-C,CeCILL-B or CeCILL license as circulated by CEA, CNRS and INRIA  
at the following URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C,CeCILL-B or CeCILL license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/

#ifndef __CALCEPHTREAD_H__
#define __CALCEPHTREAD_H__

#if HAVE_ISO_C11_THREAD
#include <threads.h>
#endif

#if HAVE_PTHREAD
#include <pthread.h>
#endif

/* the naming convention follows the ISO C11 of threads prefined by "calceph" */

#if HAVE_ISO_C11_THREAD
#define CALCEPH_HAVE_MUTEX 1
#define calceph_mtx_t  mtx_t
/* calceph_mtx_init_plain (mtx_plain) - a simple, non-recursive mutex is created. */
#define calceph_mtx_init_plain(x)  mtx_init(x, mtx_plain)
#define calceph_mtx_lock(x) mtx_lock(x)
#define calceph_mtx_unlock(x) mtx_unlock(x)
#define calceph_mtx_destroy(x) mtx_destroy(x)

#define calceph_thrd_success thrd_success
#define calceph_thrd_error thrd_error

/* define a function for calceph_thread_start_and_join */
typedef int (*thrd_start_t)(void *);

#define define_fn_thrd_start(fn,arg) static int fn(void* arg)

#elif HAVE_PTHREAD
#define CALCEPH_HAVE_MUTEX 1
#define calceph_mtx_t  pthread_mutex_t
/* calceph_mtx_init_plain (mtx_plain) - a simple, non-recursive mutex is created. */
#define calceph_mtx_init_plain(x)  pthread_mutex_init(x, NULL)
#define calceph_mtx_lock(x) pthread_mutex_lock(x)
#define calceph_mtx_unlock(x) pthread_mutex_unlock(x)
#define calceph_mtx_destroy(x) pthread_mutex_destroy(x)

#define calceph_thrd_success NULL
#define calceph_thrd_error ((void*)0x1)

typedef void *(*thrd_start_t)(void *);

#define define_fn_thrd_start(fn,arg) static void* fn(void* arg)

#elif HAVE_WIN32API
#define CALCEPH_HAVE_MUTEX 1
#define calceph_mtx_t  HANDLE
#define calceph_mtx_lock(x) WaitForSingleObject(*(x), INFINITE)
#define calceph_mtx_unlock(x) ReleaseMutex(*(x))
#define calceph_mtx_destroy(x) CloseHandle(*(x))
#define calceph_mtx_init_plain(x)  *(x) = CreateMutex(NULL, FALSE, NULL)

#define calceph_thrd_success 0
#define calceph_thrd_error 1

#define thrd_start_t LPTHREAD_START_ROUTINE
#define define_fn_thrd_start(fn,arg) static DWORD WINAPI fn(LPVOID arg)

#else
/* not available */
#define CALCEPH_HAVE_MUTEX 0
#define calceph_mtx_t  int
#define calceph_mtx_lock(x) {}
#define calceph_mtx_unlock(x) {}
#define calceph_mtx_destroy(x) {}

#define calceph_thrd_success 0
#define calceph_thrd_error 1

typedef int (*thrd_start_t)(void *);

#define define_fn_thrd_start(fn,arg) static int fn(void* arg)

#endif

__CALCEPH_DECLSPEC int calceph_thread_start_and_join(int nthreads /*<=10 */ , thrd_start_t fns[], void *args[]);

#endif
